/*****************************************************************************/
/* N.A.D.A.R. - A network tank battle game.                                  */
/*                                                                           */
/* N.A.D.A.R. Copyright (c) 2000 Sakai Hiroaki.                              */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* N.A.D.A.R. is a network tank battle game. You get into a tank,            */
/* operate it, and destroy other tanks to survive!                           */
/*****************************************************************************/

/*****************************************************************************/
/* Client 饹(Υ饹)Υ饤֥                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "ClientP.h"

/*****************************************************************************/
/* ؿ                                                                */
/*****************************************************************************/

/*===========================================================================*/
/* ѿμ                                                    */
/*===========================================================================*/

Player ClientGetPlayer(Client client) { return (client->player); }

/*===========================================================================*/
/* ֥ȤȺ                                                  */
/*===========================================================================*/

/*---------------------------------------------------------------------------*/
/* ֥Ȥ                                                        */
/*---------------------------------------------------------------------------*/

Client ClientCreate(char * hostname, unsigned short int port,
		    char * name, int size, int * argcp, char * argv[])
{
  Client client;
  Path path;
  unsigned short int command;
  unsigned short int length;

  client = (Client)malloc(sizeof(_Client));
  if (client == NULL) ErrorExit("Cannot allocate memory for client.");

  path = PathCreateToCommunicateWithServer(hostname, port,
					   CLIENT_SEND_BUFFER_SIZE,
					   CLIENT_RECV_BUFFER_SIZE);
  if (path == NULL) ErrorExit("Cannot connect to server.");

  client->player = PlayerCreate(name, path, PLAYER_ACTIVE);
  if (client->player == NULL) ErrorExit("Cannot create my player.");

  /* ɽѤΥǥץ쥤κ */
  /* 饤Ȥ can't open display ȤˡФʤ
     褦ˡФȥͥĥ˥ǥץ쥤򥪡ץ󤹤 */
  client->display = DispCreate(argcp, argv, size);
  if (client->display == NULL) ErrorExit("Cannot create display.");

  /* ФȤΥͥ */
  PathSendStringData(PlayerGetPath(client->player), JOIN_REQUEST, name);
  PathFlushSendBuffer(PlayerGetPath(client->player));

  while (!PathCheckHeader(path, &command, &length)) { usleep(10000); }
  if (command != JOIN_ACCEPT) {
    ErrorExit("failed to accept.");
  }
  PathRecvNoneData(path, &command);
  if (command != JOIN_ACCEPT) {
    ErrorExit("failed to accept.");
  }

  return (client);
}

/*---------------------------------------------------------------------------*/
/* ֥Ȥκ                                                        */
/*---------------------------------------------------------------------------*/

Client ClientDestroy(Client client)
{
  if (client == NULL) return (NULL);

  if (client->player) PlayerDestroy(client->player);

  if (client->display) DispDestroy(client->display);

  free(client);

  return (NULL);
}

/*---------------------------------------------------------------------------*/
/* 饤ȤνλȽ                                                    */
/*---------------------------------------------------------------------------*/

#if 0
static int ClientIsOver(Client client)
{
  return (0);
}
#endif

/*---------------------------------------------------------------------------*/
/* 饤ȤΥ                                                    */
/*---------------------------------------------------------------------------*/

static int ClientMain(Client client)
{
  static char buffer[300];
  unsigned short int command;
  unsigned short int length;
  unsigned long int key_press;
  unsigned long int tank_action;
  unsigned short int s1, s2, s3, s4;
  Path path;

  path = PlayerGetPath(client->player);

  while (1) {

    if (!PathCheckHeader(path, &command, &length)) {
      /*
       *  sync ʤȡư۾٤ʤ롥
       */
      DispSync(client->display);
      usleep(10000);
      return (1);
    }

    switch (command) {

    case TANK_DRAW :
      PathRecvShortShortShortShortData(path, &command, &s1, &s2, &s3, &s4);
      DispDrawTank(client->display, s1 * 0.01, s2 * 0.01, s3, s4);
      break;

    case CANON_DRAW :
      PathRecvShortShortData(path, &command, &s1, &s2);
      DispDrawCanon(client->display, s1 * 0.01, s2 * 0.01);
      break;

    case MISSILE_DRAW :
      PathRecvShortShortData(path, &command, &s1, &s2);
      DispDrawMissile(client->display, s1 * 0.01, s2 * 0.01);
      break;

    case EXPLOSION_DRAW :
      PathRecvShortShortShortData(path, &command, &s1, &s2, &s3);
      DispDrawExplosion(client->display, s1 * 0.01, s2 * 0.01, s3);
      break;

    case SEND_KEY :
      PathRecvNoneData(path, &command);
      key_press = DispGetKeyPress(client->display);
      tank_action = TANK_NO_ACTION;
      if (key_press & DISP_UP_PRESS     ) tank_action |= TANK_GO_FORWARD;
      if (key_press & DISP_DOWN_PRESS   ) tank_action |= TANK_GO_BACKWARD;
      if (key_press & DISP_LEFT_PRESS   ) tank_action |= TANK_TURN_LEFT;
      if (key_press & DISP_RIGHT_PRESS  ) tank_action |= TANK_TURN_RIGHT;
      if (key_press & DISP_BUTTON1_PRESS) tank_action |= TANK_FIRE_MISSILE;
      if (key_press & DISP_BUTTON2_PRESS) tank_action |= TANK_FIRE_CANON;
      if (key_press & DISP_BUTTON4_PRESS) tank_action |= TANK_DEAD;
      PathSendLongData(path, NOW_KEY, tank_action);
      PathFlushSendBuffer(path);
      break;

    case READ_KEY :
      PathRecvNoneData(path, &command);
      goto loop_exit;
      break;

    case DISPLAY_COPY :
      PathRecvShortShortData(path, &command, &s1, &s2);
      DispDrawDisplay(client->display, s1 * 0.01, s2 * 0.01);
      break;

    case GAME_CLEAN :
      PathRecvNoneData(path, &command);
      DispCleanGamePixmap(client->display);
      break;

    case MAP_DRAW :
      PathRecvShortShortShortData(path, &command, &s1, &s2, &s3);
      DispDrawMap(client->display, s1, s2, s3);
      break;

    case DISPLAY_CLEAR :
      PathRecvNoneData(path, &command);
      DispClearKeyPress(client->display);
      DispClearDisplay(client->display);
      break;

    case TEXT_CLEAR :
      PathRecvNoneData(path, &command);
      DispClearText(client->display);
      break;

    case TEXT_DRAW :
      PathRecvShortStringData(path, &command, &s1, buffer, 300);
      DispDrawText(client->display, s1, buffer);
      break;

    case DELETE_YOU :
      PathRecvNoneData(path, &command);
      break;

    case DELETE_ME :
      PathRecvNoneData(path, &command);
      break;

    default :
      ErrorExit("Unknown command.");
      break;
    }
  }

loop_exit :

#if 1
#if 1
  DispFlush(client->display);
#else
  /*
   *  XSync() ȡԡɤ®ʤΤ
   * ơʤˤĤƥȿߤʤȤ
   * ԲĲʸݤΤǡDispFlush() ˤ롥
   */
  DispSync(client->display);
#endif
#endif

  return (0);
}

int ClientStart(Client client)
{
  DispStart(client->display, ClientMain, client);
  return (0);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
